Class: Set
Overview
This library provides the Set class, which implements a collection of unordered values with no duplicates. It is a hybrid of Array’s intuitive inter-operation facilities and Hash’s fast lookup.
The method ‘to_set` is added to Enumerable for convenience.
Set is easy to use with Enumerable objects (implementing ‘each`). Most of the initializer methods and binary operators accept generic Enumerable objects besides sets and arrays. An Enumerable object can be converted to Set using the `to_set` method.
Set uses Hash as storage, so you must note the following points:
-
Equality of elements is determined according to Object#eql? and Object#hash. Use Set#compare_by_identity to make a set compare its elements by their identity.
-
Set assumes that the identity of each element does not change while it is stored. Modifying an element of a set will render the set to an unreliable state.
-
When a string is to be stored, a frozen copy of the string is stored instead unless the original string is already frozen.
## Comparison
The comparison operators ‘<`, `>`, `<=`, and `>=` are implemented as shorthand for the proper_,subset?,superset? methods. The `<=>` operator reflects this order, or return `nil` for sets that both have distinct elements (`y` vs. `z` for example).
## Example
“‘ruby require ’set’ s1 = Set[1, 2] #=> #<Set: 2> s2 = [1, 2].to_set #=> #<Set: 2> s1 == s2 #=> true s1.add(“foo”) #=> #<Set: 2, “foo”> s1.merge([2, 6]) #=> #<Set: 2, “foo”, 6> s1.subset?(s2) #=> false s2.subset?(s1) #=> true “‘
## Contact
-
Akinori MUSHA <<knu@iDaemons.org>> (current maintainer)
## What’s Here
First, what's elsewhere. \Class \Set:
-
Inherits from class Object.
-
Includes module Enumerable, which provides dozens of additional methods.
In particular, class Set does not have many methods of its own for fetching or for iterating. Instead, it relies on those in Enumerable.
Here, class Set provides methods that are useful for:
-
[Creating a Set](#class-Set-label-Methods+for+Creating+a+Set)
-
[Set Operations](#class-Set-label-Methods+for+Set+Operations)
-
[Comparing](#class-Set-label-Methods+for+Comparing)
-
[Querying](#class-Set-label-Methods+for+Querying)
-
[Assigning](#class-Set-label-Methods+for+Assigning)
-
[Deleting](#class-Set-label-Methods+for+Deleting)
-
[Converting](#class-Set-label-Methods+for+Converting)
-
[Iterating](#class-Set-label-Methods+for+Iterating)
-
[And more.…](#class-Set-label-Other+Methods)
### Methods for Creating a Set
-
::[]: Returns a new set containing the given objects.
-
::new: Returns a new set containing either the given objects (if no block given) or the return values from the called block (if a block given).
### Methods for Set Operations
-
[|](#method-i-7C) (aliased as #union and #+): Returns a new set containing all elements from
selfand all elements from a given enumerable (no duplicates). -
[&](#method-i-26) (aliased as #intersection): Returns a new set containing all elements common to
selfand a given enumerable. -
[-](#method-i-2D) (aliased as #difference): Returns a copy of
selfwith all elements in a given enumerable removed. -
[^](#method-i-5E): Returns a new set containing all elements from
selfand a given enumerable except those common to both.
### Methods for Comparing
-
[<=>](#method-i-3C-3D-3E): Returns -1, 0, or 1 as
selfis less than, equal to, or greater than a given object. -
[==](#method-i-3D-3D): Returns whether
selfand a given enumerable are equal, as determined by Object#eql?. -
#compare_by_identity?: Returns whether the set considers only identity when comparing elements.
### Methods for Querying
-
#length (aliased as #size): Returns the count of elements.
-
#empty?: Returns whether the set has no elements.
-
#include? (aliased as #member? and #===): Returns whether a given object is an element in the set.
-
#subset? (aliased as [<=](#method-i-3C-3D)): Returns whether a given object is a subset of the set.
-
#proper_subset? (aliased as [<](#method-i-3C)): Returns whether a given enumerable is a proper subset of the set.
-
#superset? (aliased as [>=](#method-i-3E-3D])): Returns whether a given enumerable is a superset of the set.
-
#proper_superset? (aliased as [>](#method-i-3E)): Returns whether a given enumerable is a proper superset of the set.
-
#disjoint?: Returns
trueif the set and a given enumerable have no common elements,falseotherwise. -
#intersect?: Returns
trueif the set and a given enumerable: have any common elements,falseotherwise. -
#compare_by_identity?: Returns whether the set considers only identity when comparing elements.
### Methods for Assigning
-
#add (aliased as #<<): Adds a given object to the set; returns
self. -
#add?: If the given object is not an element in the set, adds it and returns
self; otherwise, returnsnil. -
#merge: Merges the elements of each given enumerable object to the set; returns
self. -
#replace: Replaces the contents of the set with the contents of a given enumerable.
### Methods for Deleting
-
#clear: Removes all elements in the set; returns
self. -
#delete: Removes a given object from the set; returns
self. -
#delete?: If the given object is an element in the set, removes it and returns
self; otherwise, returnsnil. -
#subtract: Removes each given object from the set; returns
self. -
#delete_if - Removes elements specified by a given block.
-
#select! (aliased as #filter!): Removes elements not specified by a given block.
-
#keep_if: Removes elements not specified by a given block.
-
#reject! Removes elements specified by a given block.
### Methods for Converting
-
#classify: Returns a hash that classifies the elements, as determined by the given block.
-
#collect! (aliased as #map!): Replaces each element with a block return-value.
-
#divide: Returns a hash that classifies the elements, as determined by the given block; differs from #classify in that the block may accept either one or two arguments.
-
#flatten: Returns a new set that is a recursive flattening of
self.
\#flatten!:
Replaces each nested set in +self+ with the elements from that set.
-
#inspect (aliased as #to_s): Returns a string displaying the elements.
-
#join: Returns a string containing all elements, converted to strings as needed, and joined by the given record separator.
-
#to_a: Returns an array containing all set elements.
-
#to_set: Returns
selfif given no arguments and no block; with a block given, returns a new set consisting of block return values.
### Methods for Iterating
-
#each: Calls the block with each successive element; returns
self.
### Other Methods
-
#reset: Resets the internal state; useful if an object has been modified while an element in the set.
Constant Summary collapse
- VERSION =
"1.1.3"- InspectKey =
:nodoc:
:__inspect_key__
Class Method Summary collapse
-
.[](*ary) ⇒ Object
Creates a new set containing the given objects.
Instance Method Summary collapse
-
#&(enum) ⇒ Object
(also: #intersection)
Returns a new set containing elements common to the set and the given enumerable object.
-
#-(enum) ⇒ Object
(also: #difference)
Returns a new set built by duplicating the set, removing every element that appears in the given enumerable object.
-
#<=>(set) ⇒ Object
Returns 0 if the set are equal, -1 / +1 if the set is a proper subset / superset of the given set, or nil if they both have unique elements.
-
#==(other) ⇒ Object
Returns true if two sets are equal.
-
#^(enum) ⇒ Object
Returns a new set containing elements exclusive between the set and the given enumerable object.
-
#add(o) ⇒ Object
(also: #<<)
Adds the given object to the set and returns self.
-
#add?(o) ⇒ Boolean
Adds the given object to the set and returns self.
-
#classify ⇒ Object
Classifies the set by the return value of the given block and returns a hash of => set of elements pairs.
-
#clear ⇒ Object
Removes all elements and returns self.
-
#collect! ⇒ Object
(also: #map!)
Replaces the elements with ones returned by ‘collect()`.
-
#compare_by_identity ⇒ Object
Makes the set compare its elements by their identity and returns self.
-
#compare_by_identity? ⇒ Boolean
Returns true if the set will compare its elements by their identity.
-
#delete(o) ⇒ Object
Deletes the given object from the set and returns self.
-
#delete?(o) ⇒ Boolean
Deletes the given object from the set and returns self.
-
#delete_if(&block) ⇒ Object
Deletes every element of the set for which block evaluates to true, and returns self.
-
#disjoint?(set) ⇒ Boolean
Returns true if the set and the given enumerable have no element in common.
-
#divide(&func) ⇒ Object
Divides the set into a set of subsets according to the commonality defined by the given block.
-
#each(&block) ⇒ Object
Calls the given block once for each element in the set, passing the element as parameter.
-
#empty? ⇒ Boolean
Returns true if the set contains no elements.
-
#eql?(o) ⇒ Boolean
:nodoc:.
-
#flatten ⇒ Object
Returns a new set that is a copy of the set, flattening each containing set recursively.
-
#flatten! ⇒ Object
Equivalent to Set#flatten, but replaces the receiver with the result in place.
-
#freeze ⇒ Object
:nodoc:.
-
#hash ⇒ Object
:nodoc:.
-
#include?(o) ⇒ Boolean
(also: #member?, #===)
Returns true if the set contains the given object.
-
#initialize(enum = nil, &block) ⇒ Set
constructor
Creates a new set containing the elements of the given enumerable object.
-
#initialize_clone(orig, **options) ⇒ Object
Clone internal hash.
-
#initialize_dup(orig) ⇒ Object
Dup internal hash.
-
#inspect ⇒ Object
(also: #to_s)
Returns a string containing a human-readable representation of the set (“#<Set: element2, …>”).
-
#intersect?(set) ⇒ Boolean
Returns true if the set and the given enumerable have at least one element in common.
-
#join(separator = nil) ⇒ Object
Returns a string created by converting each element of the set to a string See also: Array#join.
-
#keep_if(&block) ⇒ Object
Deletes every element of the set for which block evaluates to false, and returns self.
-
#merge(*enums) ⇒ Object
Merges the elements of the given enumerable objects to the set and returns self.
-
#pretty_print(pp) ⇒ Object
:nodoc:.
-
#pretty_print_cycle(pp) ⇒ Object
:nodoc:.
-
#proper_subset?(set) ⇒ Boolean
(also: #<)
Returns true if the set is a proper subset of the given set.
-
#proper_superset?(set) ⇒ Boolean
(also: #>)
Returns true if the set is a proper superset of the given set.
-
#reject!(&block) ⇒ Object
Equivalent to Set#delete_if, but returns nil if no changes were made.
-
#replace(enum) ⇒ Object
Replaces the contents of the set with the contents of the given enumerable object and returns self.
-
#reset ⇒ Object
Resets the internal state after modification to existing elements and returns self.
-
#select!(&block) ⇒ Object
(also: #filter!)
Equivalent to Set#keep_if, but returns nil if no changes were made.
-
#size ⇒ Object
(also: #length)
Returns the number of elements.
-
#subset?(set) ⇒ Boolean
(also: #<=)
Returns true if the set is a subset of the given set.
-
#subtract(enum) ⇒ Object
Deletes every element that appears in the given enumerable object and returns self.
-
#superset?(set) ⇒ Boolean
(also: #>=)
Returns true if the set is a superset of the given set.
-
#to_a ⇒ Object
Returns an array containing all elements in the set.
-
#to_set(klass = Set, *args, &block) ⇒ Object
Returns self if no arguments are given.
-
#|(enum) ⇒ Object
(also: #+, #union)
Returns a new set built by merging the set and the elements of the given enumerable object.
Constructor Details
#initialize(enum = nil, &block) ⇒ Set
Creates a new set containing the elements of the given enumerable object.
If a block is given, the elements of enum are preprocessed by the given block.
Set.new([1, 2]) #=> #<Set: {1, 2}>
Set.new([1, 2, 1]) #=> #<Set: {1, 2}>
Set.new([1, 'c', :s]) #=> #<Set: {1, "c", :s}>
Set.new(1..5) #=> #<Set: {1, 2, 3, 4, 5}>
Set.new([1, 2, 3]) { |x| x * x } #=> #<Set: {1, 4, 9}>
246 247 248 249 250 251 252 253 254 255 256 |
# File 'lib/set.rb', line 246 def initialize(enum = nil, &block) # :yields: o @hash ||= Hash.new(false) enum.nil? and return if block do_with_enum(enum) { |o| add(block[o]) } else merge(enum) end end |
Class Method Details
Instance Method Details
#&(enum) ⇒ Object Also known as: intersection
646 647 648 649 650 651 652 653 654 655 656 657 658 |
# File 'lib/set.rb', line 646 def &(enum) n = self.class.new if enum.is_a?(Set) if enum.size > size each { |o| n.add(o) if enum.include?(o) } else enum.each { |o| n.add(o) if include?(o) } end else do_with_enum(enum) { |o| n.add(o) if include?(o) } end n end |
#-(enum) ⇒ Object Also known as: difference
636 637 638 |
# File 'lib/set.rb', line 636 def -(enum) dup.subtract(enum) end |
#<=>(set) ⇒ Object
Returns 0 if the set are equal, -1 / +1 if the set is a proper subset / superset of the given set, or nil if they both have unique elements.
459 460 461 462 463 464 465 466 467 |
# File 'lib/set.rb', line 459 def <=>(set) return unless set.is_a?(Set) case size <=> set.size when -1 then -1 if proper_subset?(set) when +1 then +1 if proper_superset?(set) else 0 if self.==(set) end end |
#==(other) ⇒ Object
680 681 682 683 684 685 686 687 688 689 690 |
# File 'lib/set.rb', line 680 def ==(other) if self.equal?(other) true elsif other.instance_of?(self.class) @hash == other.instance_variable_get(:@hash) elsif other.is_a?(Set) && self.size == other.size other.all? { |o| @hash.include?(o) } else false end end |
#^(enum) ⇒ Object
667 668 669 670 671 |
# File 'lib/set.rb', line 667 def ^(enum) n = self.class.new(enum) each { |o| n.add(o) unless n.delete?(o) } n end |
#add(o) ⇒ Object Also known as: <<
517 518 519 520 |
# File 'lib/set.rb', line 517 def add(o) @hash[o] = true self end |
#add?(o) ⇒ Boolean
529 530 531 |
# File 'lib/set.rb', line 529 def add?(o) add(o) unless include?(o) end |
#classify ⇒ Object
Classifies the set by the return value of the given block and returns a hash of => set of elements pairs. The block is called once for each element of the set, passing the element as parameter.
require 'set'
files = Set.new(Dir.glob("*.rb"))
hash = files.classify { |f| File.mtime(f).year }
hash #=> {2000=>#<Set: {"a.rb", "b.rb"}>,
# 2001=>#<Set: {"c.rb", "d.rb", "e.rb"}>,
# 2002=>#<Set: {"f.rb"}>}
Returns an enumerator if no block is given.
749 750 751 752 753 754 755 756 757 758 759 |
# File 'lib/set.rb', line 749 def classify # :yields: o block_given? or return enum_for(__method__) { size } h = {} each { |i| (h[yield(i)] ||= self.class.new).add(i) } h end |
#clear ⇒ Object
Removes all elements and returns self.
set = Set[1, 'c', :s] #=> #<Set: {1, "c", :s}>
set.clear #=> #<Set: {}>
set #=> #<Set: {}>
319 320 321 322 |
# File 'lib/set.rb', line 319 def clear @hash.clear self end |
#collect! ⇒ Object Also known as: map!
Replaces the elements with ones returned by ‘collect()`. Returns an enumerator if no block is given.
570 571 572 573 574 575 |
# File 'lib/set.rb', line 570 def collect! block_given? or return enum_for(__method__) { size } set = self.class.new each { |o| set << yield(o) } replace(set) end |
#compare_by_identity ⇒ Object
Makes the set compare its elements by their identity and returns self. This method may not be supported by all subclasses of Set.
260 261 262 263 264 265 266 267 |
# File 'lib/set.rb', line 260 def compare_by_identity if @hash.respond_to?(:compare_by_identity) @hash.compare_by_identity self else raise NotImplementedError, "#{self.class.name}\##{__method__} is not implemented" end end |
#compare_by_identity? ⇒ Boolean
Returns true if the set will compare its elements by their identity. Also see Set#compare_by_identity.
271 272 273 |
# File 'lib/set.rb', line 271 def compare_by_identity? @hash.respond_to?(:compare_by_identity?) && @hash.compare_by_identity? end |
#delete(o) ⇒ Object
Deletes the given object from the set and returns self. Use ‘subtract` to delete many items at once.
535 536 537 538 |
# File 'lib/set.rb', line 535 def delete(o) @hash.delete(o) self end |
#delete?(o) ⇒ Boolean
Deletes the given object from the set and returns self. If the object is not in the set, returns nil.
542 543 544 |
# File 'lib/set.rb', line 542 def delete?(o) delete(o) if include?(o) end |
#delete_if(&block) ⇒ Object
Deletes every element of the set for which block evaluates to true, and returns self. Returns an enumerator if no block is given.
549 550 551 552 553 554 555 |
# File 'lib/set.rb', line 549 def delete_if(&block) block_given? or return enum_for(__method__) { size } # Instead of directly using @hash.delete_if, perform enumeration # using self.each that subclasses may override. select(&block).each { |o| @hash.delete(o) } self end |
#disjoint?(set) ⇒ Boolean
498 499 500 |
# File 'lib/set.rb', line 498 def disjoint?(set) !intersect?(set) end |
#divide(&func) ⇒ Object
Divides the set into a set of subsets according to the commonality defined by the given block.
If the arity of the block is 2, elements o1 and o2 are in common if block.call(o1, o2) is true. Otherwise, elements o1 and o2 are in common if block.call(o1) == block.call(o2).
require 'set'
numbers = Set[1, 3, 4, 6, 9, 10, 11]
set = numbers.divide { |i,j| (i - j).abs == 1 }
set #=> #<Set: {#<Set: {1}>,
# #<Set: {11, 9, 10}>,
# #<Set: {3, 4}>,
# #<Set: {6}>}>
Returns an enumerator if no block is given.
777 778 779 780 781 782 783 784 785 786 787 788 789 790 791 792 793 794 795 796 797 798 799 800 801 802 803 804 805 |
# File 'lib/set.rb', line 777 def divide(&func) func or return enum_for(__method__) { size } if func.arity == 2 require 'tsort' class << dig = {} # :nodoc: include TSort alias tsort_each_node each_key def tsort_each_child(node, &block) fetch(node).each(&block) end end each { |u| dig[u] = a = [] each{ |v| func.call(u, v) and a << v } } set = Set.new() dig.each_strongly_connected_component { |css| set.add(self.class.new(css)) } set else Set.new(classify(&func).values) end end |
#each(&block) ⇒ Object
Calls the given block once for each element in the set, passing the element as parameter. Returns an enumerator if no block is given.
505 506 507 508 509 |
# File 'lib/set.rb', line 505 def each(&block) block_given? or return enum_for(__method__) { size } @hash.each_key(&block) self end |
#empty? ⇒ Boolean
Returns true if the set contains no elements.
310 311 312 |
# File 'lib/set.rb', line 310 def empty? @hash.empty? end |
#eql?(o) ⇒ Boolean
:nodoc:
696 697 698 699 |
# File 'lib/set.rb', line 696 def eql?(o) # :nodoc: return false unless o.is_a?(Set) @hash.eql?(o.instance_variable_get(:@hash)) end |
#flatten ⇒ Object
Returns a new set that is a copy of the set, flattening each containing set recursively.
383 384 385 |
# File 'lib/set.rb', line 383 def flatten self.class.new.flatten_merge(self) end |
#flatten! ⇒ Object
Equivalent to Set#flatten, but replaces the receiver with the result in place. Returns nil if no modifications were made.
389 390 391 |
# File 'lib/set.rb', line 389 def flatten! replace(flatten()) if any?(Set) end |
#freeze ⇒ Object
:nodoc:
298 299 300 301 |
# File 'lib/set.rb', line 298 def freeze # :nodoc: @hash.freeze super end |
#hash ⇒ Object
:nodoc:
692 693 694 |
# File 'lib/set.rb', line 692 def hash # :nodoc: @hash.hash end |
#include?(o) ⇒ Boolean Also known as: member?, ===
Returns true if the set contains the given object.
Note that include? and member? do not test member equality using == as do other Enumerables.
See also Enumerable#include?
399 400 401 |
# File 'lib/set.rb', line 399 def include?(o) @hash[o] end |
#initialize_clone(orig, **options) ⇒ Object
Clone internal hash.
293 294 295 296 |
# File 'lib/set.rb', line 293 def initialize_clone(orig, **) super @hash = orig.instance_variable_get(:@hash).clone(**) end |
#initialize_dup(orig) ⇒ Object
Dup internal hash.
287 288 289 290 |
# File 'lib/set.rb', line 287 def initialize_dup(orig) super @hash = orig.instance_variable_get(:@hash).dup end |
#inspect ⇒ Object Also known as: to_s
Returns a string containing a human-readable representation of the set (“#<Set: element2, …>”).
817 818 819 820 821 822 823 824 825 826 827 828 829 830 |
# File 'lib/set.rb', line 817 def inspect ids = (Thread.current[InspectKey] ||= []) if ids.include?(object_id) return sprintf('#<%s: {...}>', self.class.name) end ids << object_id begin return sprintf('#<%s: {%s}>', self.class, to_a.inspect[1..-2]) ensure ids.pop end end |
#intersect?(set) ⇒ Boolean
476 477 478 479 480 481 482 483 484 485 486 487 488 489 |
# File 'lib/set.rb', line 476 def intersect?(set) case set when Set if size < set.size any?(set) else set.any?(self) end when Enumerable set.any?(self) else raise ArgumentError, "value must be enumerable" end end |
#join(separator = nil) ⇒ Object
Returns a string created by converting each element of the set to a string See also: Array#join
809 810 811 |
# File 'lib/set.rb', line 809 def join(separator=nil) to_a.join(separator) end |
#keep_if(&block) ⇒ Object
Deletes every element of the set for which block evaluates to false, and returns self. Returns an enumerator if no block is given.
560 561 562 563 564 565 566 |
# File 'lib/set.rb', line 560 def keep_if(&block) block_given? or return enum_for(__method__) { size } # Instead of directly using @hash.keep_if, perform enumeration # using self.each that subclasses may override. reject(&block).each { |o| @hash.delete(o) } self end |
#merge(*enums) ⇒ Object
Merges the elements of the given enumerable objects to the set and returns self.
601 602 603 604 605 606 607 608 609 610 611 |
# File 'lib/set.rb', line 601 def merge(*enums, **nil) enums.each do |enum| if enum.instance_of?(self.class) @hash.update(enum.instance_variable_get(:@hash)) else do_with_enum(enum) { |o| add(o) } end end self end |
#pretty_print(pp) ⇒ Object
:nodoc:
834 835 836 837 838 839 840 841 842 843 |
# File 'lib/set.rb', line 834 def pretty_print(pp) # :nodoc: pp.group(1, sprintf('#<%s:', self.class.name), '>') { pp.breakable pp.group(1, '{', '}') { pp.seplist(self) { |o| pp.pp o } } } end |
#pretty_print_cycle(pp) ⇒ Object
:nodoc:
845 846 847 |
# File 'lib/set.rb', line 845 def pretty_print_cycle(pp) # :nodoc: pp.text sprintf('#<%s: {%s}>', self.class.name, empty? ? '' : '...') end |
#proper_subset?(set) ⇒ Boolean Also known as: <
Returns true if the set is a proper subset of the given set.
444 445 446 447 448 449 450 451 452 453 |
# File 'lib/set.rb', line 444 def proper_subset?(set) case when set.instance_of?(self.class) && @hash.respond_to?(:<) @hash < set.instance_variable_get(:@hash) when set.is_a?(Set) size < set.size && all?(set) else raise ArgumentError, "value must be a set" end end |
#proper_superset?(set) ⇒ Boolean Also known as: >
Returns true if the set is a proper superset of the given set.
418 419 420 421 422 423 424 425 426 427 |
# File 'lib/set.rb', line 418 def proper_superset?(set) case when set.instance_of?(self.class) && @hash.respond_to?(:>) @hash > set.instance_variable_get(:@hash) when set.is_a?(Set) size > set.size && set.all?(self) else raise ArgumentError, "value must be a set" end end |
#reject!(&block) ⇒ Object
Equivalent to Set#delete_if, but returns nil if no changes were made. Returns an enumerator if no block is given.
580 581 582 583 584 585 |
# File 'lib/set.rb', line 580 def reject!(&block) block_given? or return enum_for(__method__) { size } n = size delete_if(&block) self if size != n end |
#replace(enum) ⇒ Object
Replaces the contents of the set with the contents of the given enumerable object and returns self.
set = Set[1, 'c', :s] #=> #<Set: {1, "c", :s}>
set.replace([1, 2]) #=> #<Set: {1, 2}>
set #=> #<Set: {1, 2}>
330 331 332 333 334 335 336 337 338 339 |
# File 'lib/set.rb', line 330 def replace(enum) if enum.instance_of?(self.class) @hash.replace(enum.instance_variable_get(:@hash)) self else do_with_enum(enum) # make sure enum is enumerable before calling clear clear merge(enum) end end |
#reset ⇒ Object
Resets the internal state after modification to existing elements and returns self.
Elements will be reindexed and deduplicated.
705 706 707 708 709 710 711 712 |
# File 'lib/set.rb', line 705 def reset if @hash.respond_to?(:rehash) @hash.rehash # This should perform frozenness check. else raise FrozenError, "can't modify frozen #{self.class.name}" if frozen? end self end |
#select!(&block) ⇒ Object Also known as: filter!
Equivalent to Set#keep_if, but returns nil if no changes were made. Returns an enumerator if no block is given.
589 590 591 592 593 594 |
# File 'lib/set.rb', line 589 def select!(&block) block_given? or return enum_for(__method__) { size } n = size keep_if(&block) self if size != n end |
#size ⇒ Object Also known as: length
Returns the number of elements.
304 305 306 |
# File 'lib/set.rb', line 304 def size @hash.size end |
#subset?(set) ⇒ Boolean Also known as: <=
Returns true if the set is a subset of the given set.
431 432 433 434 435 436 437 438 439 440 |
# File 'lib/set.rb', line 431 def subset?(set) case when set.instance_of?(self.class) && @hash.respond_to?(:<=) @hash <= set.instance_variable_get(:@hash) when set.is_a?(Set) size <= set.size && all?(set) else raise ArgumentError, "value must be a set" end end |
#subtract(enum) ⇒ Object
Deletes every element that appears in the given enumerable object and returns self.
615 616 617 618 |
# File 'lib/set.rb', line 615 def subtract(enum) do_with_enum(enum) { |o| delete(o) } self end |
#superset?(set) ⇒ Boolean Also known as: >=
Returns true if the set is a superset of the given set.
405 406 407 408 409 410 411 412 413 414 |
# File 'lib/set.rb', line 405 def superset?(set) case when set.instance_of?(self.class) && @hash.respond_to?(:>=) @hash >= set.instance_variable_get(:@hash) when set.is_a?(Set) size >= set.size && set.all?(self) else raise ArgumentError, "value must be a set" end end |
#to_a ⇒ Object
345 346 347 |
# File 'lib/set.rb', line 345 def to_a @hash.keys end |
#to_set(klass = Set, *args, &block) ⇒ Object
Returns self if no arguments are given. Otherwise, converts the set to another with ‘klass.new(self, *args, &block)`.
In subclasses, returns ‘klass.new(self, *args, &block)` unless overridden.
354 355 356 357 |
# File 'lib/set.rb', line 354 def to_set(klass = Set, *args, &block) return self if instance_of?(Set) && klass == Set && block.nil? && args.empty? klass.new(self, *args, &block) end |
#|(enum) ⇒ Object Also known as: +, union
625 626 627 |
# File 'lib/set.rb', line 625 def |(enum) dup.merge(enum) end |